@@ -5,122 +5,138 @@ use 5.005;
 use strict;
 use vars qw($VERSION);
 
-use fields qw( index frames );
+use File::Spec;
 
 use overload
     '""' => \&as_string,
     fallback => 1;
 
-$VERSION = '1.10';
+$VERSION = '1.15';
+
 
 sub new
 {
     my $class = shift;
     my %p = @_;
 
-    my $self = bless { index => undef,
-		       frames => [],
-                       respect_overload => $p{respect_overload},
-		     }, $class;
+    # Backwards compatibility - this parameter was renamed to no_refs
+    # ages ago.
+    $p{no_refs} = delete $p{no_object_refs}
+        if exists $p{no_object_refs};
 
-    $self->_add_frames(%p);
+    my $self =
+        bless { index  => undef,
+                frames => [],
+                raw    => [],
+                %p,
+              }, $class;
+
+    $self->_record_caller_data();
 
     return $self;
 }
 
-sub _add_frames
+sub _record_caller_data
 {
     my $self = shift;
-    my %p = @_;
 
-    $p{no_refs} = delete $p{no_object_refs} if exists $p{no_object_refs};
+    # We exclude this method by starting one frame back.
+    my $x = 1;
+    while ( my @c =
+            do { package DB; @DB::args = (); caller($x++) } )
+    {
+        my @a = @DB::args;
+        push @{ $self->{raw} },
+            { caller => \@c,
+              args   => \@a,
+            };
+    }
+}
+
+sub _make_frames
+{
+    my $self = shift;
 
     my (@i_pack_re, %i_class);
-    if ($p{ignore_package})
+    if ( $self->{ignore_package} )
     {
-	$p{ignore_package} =
-            [$p{ignore_package}] unless UNIVERSAL::isa( $p{ignore_package}, 'ARRAY' );
+        $self->{ignore_package} =
+            [ $self->{ignore_package} ] unless UNIVERSAL::isa( $self->{ignore_package}, 'ARRAY' );
 
-        @i_pack_re = map { ref $_ ? $_ : qr/^\Q$_\E$/ } @{ $p{ignore_package} };
+        @i_pack_re = map { ref $_ ? $_ : qr/^\Q$_\E$/ } @{ $self->{ignore_package} };
     }
 
-    if ($p{ignore_class})
+    if ( $self->{ignore_class} )
     {
-	$p{ignore_class} = [$p{ignore_class}] unless ref $p{ignore_class};
-	%i_class = map {$_ => 1} @{ $p{ignore_class} };
+        $self->{ignore_class} = [ $self->{ignore_class} ] unless ref $self->{ignore_class};
+        %i_class = map {$_ => 1} @{ $self->{ignore_class} };
     }
 
     my $p = __PACKAGE__;
     push @i_pack_re, qr/^\Q$p\E$/;
 
-    my $x = 0;
-    my @c;
-    while ( do { package DB; @DB::args = (); @c = caller($x++) } )
+    my $raw = delete $self->{raw};
+    for my $r ( @{$raw} )
     {
-        next if grep { $c[0] =~ /$_/ } @i_pack_re;
-	next if grep { $c[0]->isa($_) } keys %i_class;
+        next if grep { $r->{caller}[0] =~ /$_/ } @i_pack_re;
+        next if grep { $r->{caller}[0]->isa($_) } keys %i_class;
 
-        $self->_add_frame( $p{no_refs}, \@c )
-            if @c;
+        $self->_add_frame( $r->{caller}, $r->{args} );
     }
 }
 
 sub _add_frame
 {
     my $self = shift;
-    my $no_refs = shift;
-    my $c = shift;
+    my $c    = shift;
+    my $args = shift;
 
     # eval and is_require are only returned when applicable under 5.00503.
     push @$c, (undef, undef) if scalar @$c == 6;
 
-    my @a = @DB::args;
-
-    if ( $no_refs )
+    if ( $self->{no_refs} )
     {
-        @a = map { ( ref $_
-                     ? ( UNIVERSAL::isa( $_, 'Exception::Class::Base' ) ?
-                         do { if ( $_->can('show_trace') )
-                              {
-                                  my $t = $_->show_trace;
-                                  $_->show_trace(0);
-                                  my $s = "$_";
-                                  $_->show_trace($t);
-                                  $s;
-                              }
-                              else
-                              {
-                                  # hack but should work with older
-                                  # versions of E::C::B
-                                  $_->{message};
-                              } }
-                         : $self->_ref_as_string($_)
-                       )
-                     : $_
-                   ) } @a;
+        for ( grep { ref } @{$args} )
+        {
+            # I can't remember what this is about but I think
+            # it must be to avoid a loop between between
+            # Exception::Class and this module.
+            if ( UNIVERSAL::isa( $_, 'Exception::Class::Base' ) )
+            {
+                $_ = do { if ( $_->can('show_trace') )
+                          {
+                              my $t = $_->show_trace;
+                              $_->show_trace(0);
+                              my $s = "$_";
+                              $_->show_trace($t);
+                              $s;
+                          }
+                          else
+                          {
+                              # hack but should work with older
+                              # versions of E::C::B
+                              $_->{message};
+                          }
+                        };
+            }
+            else
+            {
+                $_  = $self->_ref_as_string($_);
+            }
+        }
     }
 
     push @{ $self->{frames} },
-        Devel::StackTraceFrame->new( $c, \@a, $self->{respect_overload} );
+        Devel::StackTraceFrame->new( $c, $args, $self->{respect_overload} );
 }
 
 sub _ref_as_string
 {
     my $self = shift;
 
-    local $@;
-    if ( ref $_[0] &&
-         ! $self->{respect_overload} &&
-         eval { overload::Overloaded($_[0]) }
-       )
-    {
-        return overload::StrVal($_[0]);
-    }
-    else
-    {
-        # force stringification
-        $_[0] . '';
-    }
+    return overload::StrVal($_[0]) unless $self->{respect_overload};
+    # force stringification
+    return $_[0] . '';
 }
 
 sub next_frame
@@ -130,14 +146,15 @@ sub next_frame
     # reset to top if necessary.
     $self->{index} = -1 unless defined $self->{index};
 
-    if (defined $self->{frames}[ $self->{index} + 1 ])
+    my @f = $self->frames();
+    if ( defined $f[ $self->{index} + 1 ] )
     {
-	return $self->{frames}[ ++$self->{index} ];
+        return $f[ ++$self->{index} ];
     }
     else
     {
-	$self->{index} = undef;
-	return undef;
+        $self->{index} = undef;
+        return undef;
     }
 }
 
@@ -145,17 +162,19 @@ sub prev_frame
 {
     my $self = shift;
 
-    # reset to top if necessary.
-    $self->{index} = scalar @{ $self->{frames} } unless defined $self->{index};
+    my @f = $self->frames();
 
-    if (defined $self->{frames}[ $self->{index} - 1 ] && $self->{index} >= 1)
+    # reset to top if necessary.
+    $self->{index} = scalar @f unless defined $self->{index};
+
+    if ( defined $f[ $self->{index} - 1 ] && $self->{index} >= 1 )
     {
-	return $self->{frames}[ --$self->{index} ];
+        return $f[ --$self->{index} ];
     }
     else
     {
-	$self->{index} = undef;
-	return undef;
+        $self->{index} = undef;
+        return undef;
     }
 }
 
@@ -170,6 +189,8 @@ sub frames
 {
     my $self = shift;
 
+    $self->_make_frames() if $self->{raw};
+
     return @{ $self->{frames} };
 }
 
@@ -180,14 +201,14 @@ sub frame
 
     return unless defined $i;
 
-    return $self->{frames}[$i];
+    return ( $self->frames() )[$i];
 }
 
 sub frame_count
 {
     my $self = shift;
 
-    return scalar @{ $self->{frames} };
+    return scalar ( $self->frames() );
 }
 
 sub as_string
@@ -196,10 +217,10 @@ sub as_string
 
     my $st = '';
     my $first = 1;
-    foreach my $f (@{ $self->{frames} })
+    foreach my $f ( $self->frames() )
     {
-	$st .= $f->as_string($first) . "\n";
-	$first = 0;
+        $st .= $f->as_string($first) . "\n";
+        $first = 0;
     }
 
     return $st;
@@ -210,8 +231,6 @@ package Devel::StackTraceFrame;
 use strict;
 use vars qw($VERSION);
 
-use fields qw( package filename line subroutine hasargs wantarray evaltext is_require hints bitmask args );
-
 $VERSION = '0.6';
 
 # Create accessor routines
@@ -221,29 +240,34 @@ BEGIN
     foreach my $f ( qw( package filename line subroutine hasargs
                         wantarray evaltext is_require hints bitmask args ) )
     {
-	next if $f eq 'args';
-	*{$f} = sub { my $s = shift; return $s->{$f} };
+        next if $f eq 'args';
+        *{$f} = sub { my $s = shift; return $s->{$f} };
     }
 }
 
-sub new
 {
-    my $proto = shift;
-    my $class = ref $proto || $proto;
-
-    my $self = bless {}, $class;
-
     my @fields =
         ( qw( package filename line subroutine hasargs wantarray evaltext is_require ) );
     push @fields, ( qw( hints bitmask ) ) if $] >= 5.006;
 
-    @{ $self }{ @fields } = @{$_[0]};
+    sub new
+    {
+        my $proto = shift;
+        my $class = ref $proto || $proto;
 
-    $self->{args} = $_[1];
+        my $self = bless {}, $class;
 
-    $self->{respect_overload} = $_[2];
+        @{ $self }{ @fields } = @{$_[0]};
 
-    return $self;
+        # fixup unix-style paths on win32
+        $self->{filename} = File::Spec->canonpath( $self->{filename} );
+
+        $self->{args} = $_[1];
+
+        $self->{respect_overload} = $_[2];
+
+        return $self;
+    }
 }
 
 sub args
@@ -263,60 +287,60 @@ sub as_string
     # errors are probably my fault  -dave
     if ($first)
     {
-	$sub = 'Trace begun';
+        $sub = 'Trace begun';
     }
     else
     {
-	# Build a string, $sub, which names the sub-routine called.
-	# This may also be "require ...", "eval '...' or "eval {...}"
-	if (my $eval = $self->evaltext)
-	{
-	    if ($self->is_require)
-	    {
-		$sub = "require $eval";
-	    }
-	    else
-	    {
-		$eval =~ s/([\\\'])/\\$1/g;
-		$sub = "eval '$eval'";
-	    }
-	}
-	elsif ($sub eq '(eval)')
-	{
-	    $sub = 'eval {...}';
-	}
+        # Build a string, $sub, which names the sub-routine called.
+        # This may also be "require ...", "eval '...' or "eval {...}"
+        if (my $eval = $self->evaltext)
+        {
+            if ($self->is_require)
+            {
+                $sub = "require $eval";
+            }
+            else
+            {
+                $eval =~ s/([\\\'])/\\$1/g;
+                $sub = "eval '$eval'";
+            }
+        }
+        elsif ($sub eq '(eval)')
+        {
+            $sub = 'eval {...}';
+        }
 
-	# if there are any arguments in the sub-routine call, format
-	# them according to the format variables defined earlier in
-	# this file and join them onto the $sub sub-routine string
-	#
-	# We copy them because they're going to be modified.
-	#
-	if ( my @a = $self->args )
-	{
-	    for (@a)
-	    {
-		# set args to the string "undef" if undefined
-		$_ = "undef", next unless defined $_;
+        # if there are any arguments in the sub-routine call, format
+        # them according to the format variables defined earlier in
+        # this file and join them onto the $sub sub-routine string
+        #
+        # We copy them because they're going to be modified.
+        #
+        if ( my @a = $self->args )
+        {
+            for (@a)
+            {
+                # set args to the string "undef" if undefined
+                $_ = "undef", next unless defined $_;
 
                 # hack!
                 $_ = $self->Devel::StackTrace::_ref_as_string($_)
                     if ref $_;
 
-		s/'/\\'/g;
+                s/'/\\'/g;
 
-		# 'quote' arg unless it looks like a number
-		$_ = "'$_'" unless /^-?[\d.]+$/;
+                # 'quote' arg unless it looks like a number
+                $_ = "'$_'" unless /^-?[\d.]+$/;
 
-		# print control/high ASCII chars as 'M-<char>' or '^<char>'
-		s/([\200-\377])/sprintf("M-%c",ord($1)&0177)/eg;
-		s/([\0-\37\177])/sprintf("^%c",ord($1)^64)/eg;
-	    }
+                # print control/high ASCII chars as 'M-<char>' or '^<char>'
+                s/([\200-\377])/sprintf("M-%c",ord($1)&0177)/eg;
+                s/([\0-\37\177])/sprintf("^%c",ord($1)^64)/eg;
+            }
 
-	    # append ('all', 'the', 'arguments') to the $sub string
-	    $sub .= '(' . join(', ', @a) . ')';
-	    $sub .= ' called';
-	}
+            # append ('all', 'the', 'arguments') to the $sub string
+            $sub .= '(' . join(', ', @a) . ')';
+            $sub .= ' called';
+        }
     }
 
     return "$sub at " . $self->filename . ' line ' . $self->line;
@@ -342,13 +366,13 @@ Devel::StackTrace - Stack trace and stack trace frame objects
   # from top (most recent) of stack to bottom.
   while (my $frame = $trace->next_frame)
   {
-      print "Has args\n" if $f->hasargs;
+      print "Has args\n" if $frame->hasargs;
   }
 
   # from bottom (least recent) of stack to top.
   while (my $frame = $trace->prev_frame)
   {
-      print "Sub: ", $f->subroutine, "\n";
+      print "Sub: ", $frame->subroutine, "\n";
   }
 
 =head1 DESCRIPTION
@@ -390,7 +414,7 @@ Here's an example:
 
 =over 4
 
-=item * new(%named_params)
+=item * Devel::StackTrace->new(%named_params)
 
 Returns a new Devel::StackTrace object.
 
@@ -432,7 +456,7 @@ traces, then set this parameter to true.
 
 =back
 
-=item * next_frame
+=item * $trace->next_frame
 
 Returns the next Devel::StackTraceFrame object down on the stack.  If
 it hasn't been called before it returns the first frame.  It returns
@@ -440,7 +464,7 @@ undef when it reaches the bottom of the stack and then resets its
 pointer so the next call to C<next_frame> or C<prev_frame> will work
 properly.
 
-=item * prev_frame
+=item * $trace->prev_frame
 
 Returns the next Devel::StackTraceFrame object up on the stack.  If it
 hasn't been called before it returns the last frame.  It returns undef
@@ -448,28 +472,28 @@ when it reaches the top of the stack and then resets its pointer so
 pointer so the next call to C<next_frame> or C<prev_frame> will work
 properly.
 
-=item * reset_pointer
+=item * $trace->reset_pointer
 
 Resets the pointer so that the next call C<next_frame> or
 C<prev_frame> will start at the top or bottom of the stack, as
 appropriate.
 
-=item * frames
+=item * $trace->frames
 
 Returns a list of Devel::StackTraceFrame objects.  The order they are
 returned is from top (most recent) to bottom.
 
-=item * frame ($index)
+=item * $trace->frame ($index)
 
 Given an index, returns the relevant frame or undef if there is not
 frame at that index.  The index is exactly like a Perl array.  The
 first frame is 0 and negative indexes are allowed.
 
-=item * frame_count
+=item * $trace->frame_count
 
 Returns the number of frames in the trace object.
 
-=item * as_string
+=item * $trace->as_string
 
 Calls as_string on each frame from top to bottom, producing output
 quite similar to the Carp module's cluck/confess methods.
@@ -483,27 +507,27 @@ methods return.
 
 =over 4
 
-=item * package
+=item * $frame->package
 
-=item * filename
+=item * $frame->filename
 
-=item * line
+=item * $frame->line
 
-=item * subroutine
+=item * $frame->subroutine
 
-=item * hasargs
+=item * $frame->hasargs
 
-=item * wantarray
+=item * $frame->wantarray
 
-=item * evaltext
+=item * $frame->evaltext
 
 Returns undef if the frame was not part of an eval.
 
-=item * is_require
+=item * $frame->is_require
 
 Returns undef if the frame was not part of a require.
 
-=item * args
+=item * $frame->args
 
 Returns the arguments passed to the frame.  Note that any arguments
 that are references are returned as references, not copies.
@@ -514,18 +538,29 @@ that are references are returned as references, not copies.
 
 =over 4
 
-=item * hints
+=item * $frame->hints
 
-=item * bitmask
+=item * $frame->bitmask
 
 =back
 
+=head1 SUPPORT
+
+Please submit bugs to the CPAN RT system at
+http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Devel%3A%3AStackTrace
+or via email at bug-devel-stacktrace@rt.cpan.org.
+
 =head1 AUTHOR
 
 Dave Rolsky, <autarch@urth.org>
 
-=head1 SEE ALSO
+=head1 COPYRIGHT
 
-Exception::Class
+Copyright (c) 2000-2006 David Rolsky.  All rights reserved.  This
+program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
+
+The full text of the license can be found in the LICENSE file included
+with this module.
 
 =cut
